{
  DX8D3DX - DirectX 8 D3DX unit.
  Copyright  2001 Lord Trancos.
  <lordtrancos@softhome.net>
}

unit DX8D3DX;

// --------------------------------------------------------------------

interface

uses Windows, SysUtils, Classes, DirectXGraphics, DX8GFX, D3DX8;

var
  D3DXSPR8 : ID3DXSprite = NIL;

type
  // Z-Sprite - Used to load and draw large pictures
  //            (> device texture size).
  TZSpr = record
             Texts  : array of IDirect3DTexture8;
             Width  : longint; // Total img Width.
             Height : longint; // Total img Height.
             xCRes  : longint; // Cell X Y
             yCRes  : longint; // Resolution.
             xCells : longint; // # X Cells.
             yCells : longint; // # Y Cells.
           end;

// Main functions
function  InitD3DX: boolean;
procedure CloseD3DX;

// Surface functions
function  D3DXGetPicInfo(_filename: pChar;
                         var _nfo: TD3DXImageInfo): boolean;
function  D3DXLoadSurfaceFromFile(_filename: pChar;
                                  var _width, _height: longint;
                                  _format: TD3DFormat): IDirect3DSurface8;
procedure D3DXSurfToTextBitBlt(_surf: IDirect3DSurface8;
                               _surfWidth,
                               _srcOffsetX, _srcOffsetY,
                               _dstOffsetX, _dstOffsetY,
                               _dstWidth, _dstHeight : longint;
                               _text: IDirect3DTexture8);

// Texture/Sprite functions
function  D3DXLoadTextureFromFile(_filename: pChar;
                                  _cKey: TD3DColor;
                                  _format: TD3DFormat;
                                  _mipLevels: word): IDirect3DTexture8;
procedure D3DXDrawSpr(_texture: IDirect3DTexture8; _srcRect: PRect;
                      _translation, _scaling, _rotAxis: PD3DXVector2;
                      _rotation: single; _rgb: longint; _alfa: byte);

// Z-Sprite functions
procedure D3DXResetZSpr(var _zSpr: TZSpr);
procedure D3DXFreeZText(var _zSpr: TZSpr);
function  D3DXLoadZSpr(_filename: pChar; _format: TD3DFormat;
                       var _zSpr: TZSpr): boolean;
procedure D3DXDrawZSpr(_zSpr: TZSpr;
                       _translation, _scaling: TD3DXVector2;
                       _rgb: longint; _alfa: byte);

// Font
function D3DXCreateFont(_name: pChar; _height: longint;
                        _bold, _italic, _underline: boolean): ID3DXFont;
function D3DXFontWrite(_fnt: ID3DXFont; _text: string; _rect: TRect;
                       _format: cardinal; _color: TD3DColor): boolean;

// --------------------------------------------------------------------

implementation

// --------------------------------------------------------------------
// Name : InitD3DX.
// Desc : Initializes D3DX Interfaces.
// --------------------------------------------------------------------

function  InitD3DX: boolean;

begin
  Result := false;

  // Get a ID3DXSprite interface.
  if failed(D3DXCreateSprite(D3DDEV8, D3DXSPR8)) then exit;

  // All right.
  Result := true;
end;

// --------------------------------------------------------------------
// Name : CloseD3DX.
// Desc : Closes D3DX Interfaces.
// --------------------------------------------------------------------

procedure CloseD3DX;

begin
  if assigned(D3DXSPR8) then D3DXSPR8 := NIL;
end;

// --------------------------------------------------------------------
// Name : D3DXGetPicInfo.
// Desc : Loads a picture, retrieves some information about it, and
//        frees the picture. 
// --------------------------------------------------------------------

function D3DXGetPicInfo(_filename: pChar;
                        var _nfo: TD3DXImageInfo): boolean;

var _surf : IDirect3DSurface8;
    _r : TRect;

begin
  Result := false;

  // Create a 16 bit dummy surface.
  _surf := NIL;
  if (failed(D3DDEV8.CreateImageSurface(4, 4, D3DFMT_R5G6B5,
             _surf))) or (_surf = NIL) then exit;

  // Load file into dummy surface.
  _r := Rect(0, 0, 3, 3);
  if (failed(D3DXLoadSurfaceFromFileA(_surf, NIL, @_r, _filename, NIL,
                           D3DX_FILTER_NONE, 0, @_nfo)))
    then exit;

  // Free surface,
  _surf := NIL;

  // All right.
  Result := true;
end;

// --------------------------------------------------------------------
// Name : D3DXLoadSurfaceFromFile.
// Desc : Loads a surface from a file (bmp, jpeg, png...)
//        _format must be 0 to use the default image format.
// --------------------------------------------------------------------

function D3DXLoadSurfaceFromFile(_filename: pChar;
                                 var _width, _height: longint;
                                 _format: TD3DFormat): IDirect3DSurface8;

var _imginfo : TD3DXImageInfo;

begin
  Result := NIL;

  // Get pic nfo
  if not D3DXGetPicInfo(_filename, _imginfo) then exit;
  _width  := _imginfo.Width;
  _height := _imginfo.Height;

  // Create the surface.
  if _format = 0 then _format := _imginfo.Format;
  with _imginfo do
    if (failed(D3DDEV8.CreateImageSurface(Width, Height, _format,
        Result))) or (Result = NIL) then exit;

  // Load file into the surface.
  if (failed(D3DXLoadSurfaceFromFileA(Result, NIL, NIL, _filename,
                          NIL, D3DX_FILTER_NONE, 0, NIL)))
    then Result := NIL;
end;

// --------------------------------------------------------------------
// Name : D3DXSurfToTextBitBlt.
// Desc : Copies data from surfate to texture.
//        Surface and texture must have the same format.
//        Warning : this function does not perform cliping.
//                  (Be careful using this function...)
// --------------------------------------------------------------------

procedure D3DXSurfToTextBitBlt(_surf: IDirect3DSurface8;
                               _surfWidth,
                               _srcOffsetX, _srcOffsetY,
                               _dstOffsetX, _dstOffsetY,
                               _dstWidth, _dstHeight : longint;
                               _text: IDirect3DTexture8);

var _sr, _tr : TD3DLocked_Rect;
    _pixelSize : longint; // Pixel size in bytes.
    _scanlSize : longint; // Destination scan line size in bytes.
    _srcCur : pByte;
    _dstCur : pByte;
    _cntY : longint;

begin
  // Copy data
  if (not failed(_surf.LockRect(_sr, NIL, 0))) and
     (not failed(_text.LockRect(0, _tr, NIL, 0))) then
    begin
      _pixelSize := _sr.Pitch div _surfWidth;
      _scanlSize := _pixelSize * _dstWidth;

      // Set src cursor.
      _srcCur := _sr.pBits;
      inc(_srcCur, (_sr.Pitch * _srcOffsetY) + (_pixelSize * _srcOffsetX));

      // Set dst cursor.
      _dstCur := _tr.pBits;
      inc(_dstCur, (_tr.Pitch * _dstOffsetY) + (_pixelSize * _dstOffsetX));

      // Copy Data.
      for _cntY := 0 to _dstHeight - 1 do
        begin
          Move(_srcCur^, _dstCur^, _scanlSize);
          // Next line.
          inc(_srcCur, _sr.Pitch);
          inc(_dstCur, _tr.Pitch);
        end;
    end;

  // Unlock.
  _text.UnlockRect(0);
  _surf.UnlockRect;
end;

// --------------------------------------------------------------------
// Name : D3DXLoadTextureFromFile.
// Desc : Loads a texture from a file (bmp, jpeg, png...)
//        _cKey -> ColorKey (ARGB format, 0 = none, $FF000000 = black,
//                 $FFFF0000 = red, $FF00FF00 = green, etc.)
//        _format must be 0 to use the default image format.
// --------------------------------------------------------------------

function D3DXLoadTextureFromFile(_filename: pChar;
                                 _cKey: TD3DColor;
                                 _format: TD3DFormat;
                                 _mipLevels: word): IDirect3DTexture8;

begin
  Result := NIL;
  if failed(D3DXCreateTextureFromFileExA(D3DDEV8, _filename, 0, 0,
                                      _mipLevels, 0, _format,
                                      D3DPOOL_DEFAULT, D3DX_FILTER_NONE,
                                      D3DX_FILTER_NONE, _cKey, NIL, NIL,
                                      Result))
    then Result := NIL;
end;

// --------------------------------------------------------------------
// Name : D3DXDrawSpr.
// Desc : Draws a "2D" sprite.
//        Warning: "Drawing" outside the screen can freeze Windows...
// --------------------------------------------------------------------

procedure D3DXDrawSpr(_texture: IDirect3DTexture8; _srcRect: PRect;
                      _translation, _scaling, _rotAxis: PD3DXVector2;
                      _rotation: single; _rgb: longint; _alfa: byte);

begin
  D3DXSPR8.Draw(_texture, _srcRect, _scaling, _rotAxis, _rotation,
                _translation, (_alfa shl 24) + _rgb);
end;

// --------------------------------------------------------------------
// Name : D3DXResetZSpr.
// Desc : Clears a Z-Sprite structure.
// --------------------------------------------------------------------

procedure D3DXResetZSpr(var _zSpr: TZSpr);

begin
  FillChar(_zSpr, SizeOf(_zSpr), 0);
end;

// --------------------------------------------------------------------
// Name : D3DXFreeZText.
// Desc : Call this function to release memory used by a Z-Sprite.
// --------------------------------------------------------------------

procedure D3DXFreeZText(var _zSpr: TZSpr);

var _cnt : longint;

begin
  // Free cells.
  if (length(_zSpr.Texts) > 0) then
    for _cnt := 0 to length(_zSpr.Texts) - 1 do
      if Assigned(_zSpr.Texts[_cnt]) then _zSpr.Texts[_cnt] := NIL;
  // Free array.
  _zSpr.Texts := NIL;
  // Clear data.
  FillChar(_zSpr, SizeOf(_zSpr), 0);
end;

// --------------------------------------------------------------------
// Name : D3DXLoadZSpr.
// Desc : Loads a picture of any size. If it's bigger than the current
//        device max texture size, the image it's splited.
// --------------------------------------------------------------------

function D3DXLoadZSpr(_filename: pChar; _format: TD3DFormat;
                      var _zSpr: TZSpr): boolean;

var _surf : IDirect3DSurface8;
    _width, _height : longint;
    _cntX, _cntY : longint;
    _dw, _dh : longint;
    _cnt : longint;

begin
  Result := false;

  // Load pic as a surface.
  _surf := D3DXLoadSurfaceFromFile(_filename, _width, _height, _format);
  if _surf = NIL then exit;

  // Get resolution.
  _zSpr.Width  := _width;
  _zSpr.Height := _height;

  // Check how many textures are requiered for current device.
  _zSpr.xCRes  := D3DDEVCAPS8.MaxTextureWidth;
  _zSpr.xCells := 0;
  _cnt := _width;
  repeat
    inc(_zSpr.xCells);
    dec(_cnt, _zSpr.xCRes);
  until (_cnt <= 0);

  _zSpr.yCRes  := D3DDEVCAPS8.MaxTextureHeight;
  _zSpr.yCells := 0;
  _cnt := _height;
  repeat
    inc(_zSpr.yCells);
    dec(_cnt, _zSpr.yCRes);
  until (_cnt <= 0);

  // Create those textures
  try
    SetLength(_zSpr.Texts, _zSpr.xCells * _zSpr.yCells);
  except
    _surf := NIL;
    exit;
  end;

  for _cnt := 0 to Length(_zSpr.Texts)-1 do
    begin
      if failed (D3DXCreateTexture(D3DDEV8, D3DDEVCAPS8.MaxTextureWidth,
                                   D3DDEVCAPS8.MaxTextureHeight, 1,
                                   0, _format, D3DPOOL_MANAGED,
                                   _zSpr.Texts[_cnt])) then
        begin
          D3DXFreeZText(_zSpr);
          _surf := NIL;
          exit;
        end;
    end;

  // copy surface data to textures.
  _cnt := 0;
  for _cntY := 0 to _zSpr.yCells - 1 do
    for _cntX := 0 to _zSpr.xCells - 1 do
      begin
        if ((_cntX+1) * _zSpr.xCRes) > _zSpr.Width
          then _dw := _zSpr.Width - (_cntX * _zSpr.xCRes)
            else _dw := _zSpr.xCRes;

        if ((_cntY+1) * _zSpr.yCRes) > _zSpr.Height
          then _dh := _zSpr.Height - (_cntY * _zSpr.yCRes)
            else _dh := _zSpr.yCRes;

        D3DXSurfToTextBitBlt(_surf, _width,
                             _cntX * _zSpr.xCRes, _cntY * _zSpr.yCRes,
                             0, 0, _dw, _dh, _zSpr.Texts[_cnt]);

        // next cell
        inc(_cnt);
      end;

  // All right.
  Result := true;
end;

// --------------------------------------------------------------------
// Name : D3DXDrawZSpr.
// Desc : Draws a Z-Sprite.
// --------------------------------------------------------------------

procedure D3DXDrawZSpr(_zSpr: TZSpr;
                       _translation, _scaling: TD3DXVector2;
                       _rgb: longint; _alfa: byte);

var _cntX, _cntY : longint;
    _trans   : TD3DXVector2;
    _dw, _dh : longint;
    _srcRect : TRect;
    _cnt : longint;

begin
  _cnt := 0;
  if (length(_zSpr.Texts) > 0) then
    for _cntY := 0 to _zSpr.yCells-1 do
      for _cntX := 0 to _zSpr.xCells-1 do
        begin
          if ((_cntX+1) * _zSpr.xCRes) > _zSpr.Width
            then _dw := _zSpr.Width - (_cntX * _zSpr.xCRes)
              else _dw := _zSpr.xCRes;

          if ((_cntY+1) * _zSpr.yCRes) > _zSpr.Height
            then _dh := _zSpr.Height - (_cntY * _zSpr.yCRes)
              else _dh := _zSpr.yCRes;

          _srcRect := Rect(0, 0, _dw, _dh);
          _trans.x := _translation.x + (_cntX * _zSpr.xCRes * _scaling.x);
          _trans.y := _translation.y + (_cntY * _zSpr.yCRes * _scaling.y);

          // Draw only if it's on screen!!
          if (_trans.x + _dw*_scaling.x >= 0) and
             (_trans.y + _dh*_scaling.y >= 0) and
             (_trans.x < D3DDEVPPAR8.BackBufferWidth) and
             (_trans.y < D3DDEVPPAR8.BackBufferHeight) then
            D3DXSPR8.Draw(_zSpr.Texts[_cnt], @_srcRect, @_scaling, NIL, 0,
                          @_trans, (_alfa shl 24) + _rgb);

          // next cell
          inc(_cnt);
        end;
end;

// --------------------------------------------------------------------

function D3DXCreateFont(_name: pChar; _height: longint;
                        _bold, _italic, _underline: boolean): ID3DXFont;

var _lf : TLogFont;

begin
  FillChar(_lf, SizeOf(_lf), 0);
  with _lf do
  begin
    if (_bold) then lfWeight := FW_BOLD;
    if (_italic) then lfItalic := 1;
    if (_underline) then lfUnderline := 1;
    lfHeight := _height;
    move(_name^, lfFaceName, strLen(_name));
  end;

  Result := NIL;
  if failed(D3DXCreateFontIndirect(D3DDEV8, _lf, Result))
  then Result := NIL;
end;

// --------------------------------------------------------------------

function D3DXFontWrite(_fnt: ID3DXFont; _text: string; _rect: TRect;
                       _format: cardinal; _color: TD3DColor): boolean;

begin
  Result := not failed(_fnt.DrawTextA(pChar(_text), length(_text),
                                      _rect, _format, _color));
end;

// --------------------------------------------------------------------

end.
