{
  DX8GUI3D - DirectX 8 3D GUI unit.
  Copyright  2001 Lord Trancos.
  <lordtrancos@softhome.net>
}

unit DX8GUI3D;

// --------------------------------------------------------------------

interface

uses Windows, DirectXGraphics, DX8GFX;

const
  D3DFVF_GUI3D = D3DFVF_XYZRHW or D3DFVF_DIFFUSE or D3DFVF_TEX1;

type
  TGUI3DVertex = record
                   X, Y, Z, RHW : single;
                   DColor : dWord;
                   U, V : single;
                 end;
  PGUI3DVertex = ^TGUI3DVertex;

  TGUI3DColors = array[0..3] of dWord;

  TUVMapMode  = (uvNormal);

var
  GUI3D_RectVB : IDirect3DVertexBuffer8 = NIL;
  GUI3D_RectVA : array[0..3] of TGUI3DVertex;

// Main functions
function  InitGUI3D: boolean;
procedure CloseGUI3D;

// Color functions
function gui3dOneColor(_argb: dWord): TGUI3DColors;
function gui3dFourColors(_c1, _c2, _c3, _c4: dWord): TGUI3DColors;

// Render state / Texture Stage State functions.
procedure guid3dConfigA;
procedure guid3dConfigB;
procedure guid3dConfigC;
procedure guid3dConfigD;
procedure guid3dConfigE;

// Draw functions
procedure guid3dBeginDraw;
function  guid3dDrawRect(_rect: TRect; _colors: TGUI3DColors;
                         _uvMap: TUVMapMode; _uvScale: single): boolean;

// --------------------------------------------------------------------

implementation

// --------------------------------------------------------------------

function InitGUI3D: boolean;

var _cnt : longint;

begin
  // Create a vertex buffer that will be used for drawing rects.
  Result := not failed(D3DDEV8.CreateVertexBuffer(
                                   SizeOf(GUI3D_RectVA),
                                   D3DUSAGE_DYNAMIC or D3DUSAGE_WRITEONLY,
                                   D3DFVF_GUI3D, D3DPOOL_DEFAULT,
                                   GUI3D_RectVB));

  // Set default values for Z and RHW
  for _cnt := 0 to 3 do
  begin
    GUI3D_RectVA[_cnt].Z := 0.5;
    GUI3D_RectVA[_cnt].RHW := 1;
  end;
end;

// --------------------------------------------------------------------

procedure CloseGUI3D;

begin
  // Free vertex buffer.
  if Assigned(GUI3D_RectVB) then GUI3D_RectVB := NIL;
end;

// --------------------------------------------------------------------

function gui3dOneColor(_argb: dWord): TGUI3DColors;

var _cnt : longint;

begin
  for _cnt := 0 to 3 do Result[_cnt] := _argb;
end;

// --------------------------------------------------------------------

function gui3dFourColors(_c1, _c2, _c3, _c4: dWord): TGUI3DColors;

begin
  Result[0] := _c1;
  Result[1] := _c3;
  Result[2] := _c2;
  Result[3] := _c4;
end;

// --------------------------------------------------------------------

procedure guid3dBeginDraw;

begin
  // Set the stream source.
  D3DDEV8.SetStreamSource(0, GUI3D_RectVB, sizeof(TGUI3DVertex));

  // Set the vertex shader.
  D3DDEV8.SetVertexShader(D3DFVF_GUI3D);
end;

// --------------------------------------------------------------------

procedure guid3dConfigA;

// Color Key

begin
  D3DDEV8.SetRenderState(D3DRS_ALPHAREF, 1);
  D3DDEV8.SetRenderState(D3DRS_ALPHAFUNC, D3DCMP_GREATEREQUAL);
  D3DDEV8.SetRenderState(D3DRS_ALPHATESTENABLE, 1);
  D3DDEV8.SetTextureStageState(0, D3DTSS_COLOROP,   D3DTOP_SELECTARG1);
  D3DDEV8.SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
  D3DDEV8.SetTextureStageState(0, D3DTSS_ALPHAOP,   D3DTOP_SELECTARG1);
  D3DDEV8.SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
end;

// --------------------------------------------------------------------

procedure guid3dConfigB;

// Color Key + Texure-Diffuse Alpha Blend

begin
  D3DDEV8.SetRenderState(D3DRS_ALPHAREF, 1);
  D3DDEV8.SetRenderState(D3DRS_ALPHAFUNC, D3DCMP_GREATEREQUAL);
  D3DDEV8.SetRenderState(D3DRS_ALPHATESTENABLE, 1);
  D3DDEV8.SetRenderState(D3DRS_ALPHABLENDENABLE, 1);
  D3DDEV8.SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
  D3DDEV8.SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
  D3DDEV8.SetTextureStageState(0, D3DTSS_COLOROP,   D3DTOP_SELECTARG1);
  D3DDEV8.SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
  D3DDEV8.SetTextureStageState(0, D3DTSS_ALPHAOP,   D3DTOP_MODULATE);
  D3DDEV8.SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
  D3DDEV8.SetTextureStageState(0, D3DTSS_ALPHAARG2, D3DTA_DIFFUSE);
end;

// --------------------------------------------------------------------

procedure guid3dConfigC;

// No colorkey + Diffuse Alpha Blend

begin
  D3DDEV8.SetRenderState(D3DRS_ALPHATESTENABLE, 0);
  D3DDEV8.SetTextureStageState(0, D3DTSS_COLOROP,   D3DTOP_SELECTARG1);
  D3DDEV8.SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_DIFFUSE);
  D3DDEV8.SetTextureStageState(0, D3DTSS_ALPHAOP,   D3DTOP_SELECTARG1);
  D3DDEV8.SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_DIFFUSE);
  D3DDEV8.SetRenderState(D3DRS_ALPHABLENDENABLE, 1);
  D3DDEV8.SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
  D3DDEV8.SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
end;

// --------------------------------------------------------------------

procedure guid3dConfigD;

// No colorkey + Texture Alpha Blend

begin
  D3DDEV8.SetRenderState(D3DRS_ALPHATESTENABLE, 0);
  D3DDEV8.SetTextureStageState(0, D3DTSS_COLOROP,   D3DTOP_SELECTARG1);
  D3DDEV8.SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
  D3DDEV8.SetTextureStageState(0, D3DTSS_ALPHAOP,   D3DTOP_SELECTARG1);
  D3DDEV8.SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
  D3DDEV8.SetRenderState(D3DRS_ALPHABLENDENABLE, 1);
  D3DDEV8.SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
  D3DDEV8.SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
end;

// --------------------------------------------------------------------

procedure guid3dConfigE;

// No colorkey + No Alpha Blend

begin
  D3DDEV8.SetRenderState(D3DRS_ALPHATESTENABLE, 0);
  D3DDEV8.SetRenderState(D3DRS_ALPHABLENDENABLE, 0);
  D3DDEV8.SetTextureStageState(0, D3DTSS_COLOROP,   D3DTOP_MODULATE);
  D3DDEV8.SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
  D3DDEV8.SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);
end;

// --------------------------------------------------------------------

function guid3dDrawRect(_rect: TRect; _colors: TGUI3DColors;
                        _uvMap: TUVMapMode; _uvScale: single): boolean;

var _cnt : longint;
    _ptr : pByte;

begin
  Result := false;

  // Set rect
  GUI3D_RectVA[0].X := _rect.Left;
  GUI3D_RectVA[0].Y := _rect.Top;
  GUI3D_RectVA[1].X := _rect.Left;
  GUI3D_RectVA[1].Y := _rect.Bottom;
  GUI3D_RectVA[2].X := _rect.Right;
  GUI3D_RectVA[2].Y := _rect.Top;
  GUI3D_RectVA[3].X := _rect.Right;
  GUI3D_RectVA[3].Y := _rect.Bottom;

  // Set colors
  for _cnt := 0 to 3 do GUI3D_RectVA[_cnt].dColor := _colors[_cnt];

  // Set UV Map
  _uvScale := 1 / _uvScale;  
//  case _uvMap of
  GUI3D_RectVA[0].U := 0;         GUI3D_RectVA[0].V := 0;
  GUI3D_RectVA[1].U := 0;         GUI3D_RectVA[1].V := _uvScale;
  GUI3D_RectVA[2].U := _uvScale;  GUI3D_RectVA[2].V := 0;
  GUI3D_RectVA[3].U := _uvScale;  GUI3D_RectVA[3].V := _uvScale;
//  end;

  // Lock, fill and unlock vertex buffer.
  if failed(GUI3D_RectVB.Lock(0, SizeOf(GUI3D_RectVA), _ptr,
                              D3DLOCK_DISCARD)) then exit;
  Move(GUI3D_RectVA, _ptr^, SizeOf(GUI3D_RectVA));
  GUI3D_RectVB.Unlock;

  // Draw
  D3DDEV8.DrawPrimitive(D3DPT_TRIANGLESTRIP, 0, 2);

  // All right
  Result := true;
end;

// --------------------------------------------------------------------

end.
