//
// GUI Sample - by Lord Trancos
// ----------------------------
//
// This sample shows;
//
//   - Easy rect draw.
//   - Text draw.
//   - Mouse cursor draw.
//   - Alpha Blending.
//   - ColorKey.
//   - and Fade Out.
//
// Thanks to Gilles van den Hoven and LoreKeeper.
//

unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls,
  Forms, Dialogs, DirectXGraphics, D3DX8,
  DX8GFX, DX8D3DX, DX8GUI3D, DX8DI;

type
  TForm1 = class(TForm)
    procedure hHalt(_errMsg: pChar);
    procedure AppOnIdle(Sender: TObject; var Done: Boolean);
    procedure AppOnActivate(Sender: TObject);
    procedure AppOnDeactivate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

  FRAMECONTROL  : dWord;
  LASTFRAMETIME : dWord;
  DXReady      : boolean = false;
  WindowReady  : boolean = false;

  VertexBuffer : IDirect3DVertexBuffer8 = NIL;
  Texture1     : IDirect3DTexture8 = NIL; // Clouds
  Texture2     : IDirect3DTexture8 = NIL; // Cursor

  // Fonts
  Font1 : ID3DXFont = NIL;
  Font2 : ID3DXFont = NIL;

  // Matrices
  matWorld : TD3DXMatrix; // World
  matView  : TD3DXMatrix; // View
  matProj  : TD3DXMatrix; // Projection

  // Cursor position
  CursorPos : TPoint;

  // -
  SwapS : boolean = false;
  CountDown : longint;

type
  // The structure of the custom vertex.
  PCustomVertex = ^TCustomVertex;
  TCustomVertex = record
                    Position : TD3DXVector3; // Position.
                    DColor   : TD3DColor;    // Diffuse Color.
                    U, V     : single;       // Texture coordinates.
                  end;

const
  // Flexible vector format (FVF) description.
  // Describes the custom vertex structure (TCustomVertex).
  D3DFVF_CUSTOMVERTEX = D3DFVF_XYZ or D3DFVF_DIFFUSE or D3DFVF_TEX1;

  // Texture filtering.
  USE_TEXTURE_FILTERING = true;

const
  FADE_TIME = 1000; // msec
  TEXTFILENAME1 = 'CLOUDS.PNG';
  TEXTFILENAME2 = 'CURSOR.PNG';
  WRN00 = 'Error initializing 3D Hardware.'#10+
          'Now software will be used...';
  ERR00 = 'Could not initialize DirectX Graphics.'#10+
          'You must have installed DirectX 8 or better '#10+
          'and a 16, 24 or 32 bpp desktop to run this sample.';
  ERR01 = 'Error initializing Vertex Buffer.';
  ERR02 = 'Error loading file: ';
  ERR03 = 'Error initializing DirectInput.'#10+
          'You must have installed DirectX 8 or better.';
  ERR04 = 'Error initializing mouse.';
  ERR05 = 'Error creating font.';
  ERR06 = 'Error initializing GUI3D.';

implementation

{$R *.DFM}

// -------------------------------------------------------------------

procedure GetUserInput;

var _x, _y, _0u, _1u, _z : longint;

begin
  if DIGetMouseState(_x, _y, _0u, _z, _z, _1u, _z, _z) then
    begin
      if (_0u > 0) then SwapS := not SwapS;
      if (_1u > 0) then CountDown := FADE_TIME; // msec.
      with CursorPos do
      begin
        X := X + _x;
        Y := Y + _y;
        if (X < 0) then X := 0;
        if (Y < 0) then Y := 0;
        if (X >= 640) then X := 640-1;
        if (Y >= 480) then Y := 480-1;
      end;
    end else DIMouseControl(true);
end;

// -------------------------------------------------------------------

procedure SetupMatrices;

var _v1, _v2, _v3 : TD3DXVector3;

begin
  // World.
  D3DXMatrixRotationX(matWorld, sin(GetTickCount/1000) * 1.0);
  D3DDEV8.SetTransform(D3DTS_WORLD, matWorld);

  // View.
  _v1 := D3DXVector3( 0.0, 0.0, -5.0);
  _v2 := D3DXVector3( 0.0, 0.0,  0.0);
  _v3 := D3DXVector3( 0.0, 1.0,  0.0);
  D3DXMatrixLookAtLH(matView, _v1, _v2, _v3);
  D3DDEV8.SetTransform(D3DTS_VIEW, matView);

  // Projection.
  D3DXMatrixPerspectiveFovLH(matProj, PI/4, 1.0, 1.0, 100.0);
  D3DDEV8.SetTransform(D3DTS_PROJECTION, matProj);
end;

// -------------------------------------------------------------------

procedure RenderFrame;

var _fade : longint;

begin
  // Clear the back buffer to a black color.
  D3DDEV8.Clear(0, NIL, D3DCLEAR_TARGET or D3DCLEAR_ZBUFFER,
                $FF000080, 1.0, 0);

  // Begin the scene.
  D3DDEV8.BeginScene;

  // Set Transform states.
  SetupMatrices;

  // Set the stream source.
  D3DDEV8.SetStreamSource(0, VertexBuffer, sizeof(TCustomVertex));

  // Set the vertex shader.
  D3DDEV8.SetVertexShader(D3DFVF_CUSTOMVERTEX);

  // Set the texture that the will be used to render.
  D3DDEV8.SetTexture(0, Texture1);

  // Set the texture stage state values.
  D3DDEV8.SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_MODULATE);
  D3DDEV8.SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
  D3DDEV8.SetTextureStageState( 0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);
  D3DDEV8.SetTextureStageState( 0, D3DTSS_ALPHAOP,   D3DTOP_DISABLE);

  // Render the vertices.
  D3DDEV8.DrawPrimitive(D3DPT_TRIANGLESTRIP, 0, 98);

  { ****** Draw GUI ****** }
  guid3dBeginDraw;

  if (not SwapS) then
  begin
    // Draw white grad.
    guid3dConfigC;
    D3DDEV8.SetTexture(0, NIL);
    guid3dDrawRect(Rect(100, 100, 540, 180),
                   gui3dFourColors($FFFFFFFF, $00FF0000,
                                   $000000FF, $FFFFFFFF), uvNormal, 1);

    // Draw yellow rect.
    guid3dDrawRect(Rect(100, 200, 540, 280),
                   gui3dOneColor($80FFFF00), uvNormal, 1);

    // Draw clouds rect.
    guid3dConfigD;
    D3DDEV8.SetTexture(0, Texture1);
    guid3dDrawRect(Rect(100, 300, 540, 380),
                   gui3dFourColors($FFFFFFFF, $00000000,
                                   $00000000, $FFFFFFFF), uvNormal, 1);

    // Draw yellow-red grad.
    guid3dConfigE;
    D3DDEV8.SetTexture(0, NIL);
    guid3dDrawRect(Rect(200, 220, 440, 260),
                   gui3dFourColors($FFFFFF00, $FFFF0000,
                                   $FFFFFF00, $FFFF0000), uvNormal, 1);

    // Cloud Textured Yellow-red grad.
    D3DDEV8.SetTexture(0, Texture1);
    guid3dDrawRect(Rect(200, 320, 440, 360),
                   gui3dFourColors($FFFFFF00, $FFFF0000,
                                   $FFFFFF00, $FFFF0000), uvNormal, 1);

    // Draw text
    Font1._Begin;
    D3DXFontWrite(Font1, 'Click to change!', Rect(225, 225, 500, 300),
                  0, $FF000000);
    D3DXFontWrite(Font1, 'Click to change!', Rect(220, 220, 500, 300),
                  0, $FFFFFFFF);
    Font1._End;

    // Draw cursor
    guid3dConfigA;
    D3DDEV8.SetTexture(0, Texture2);
    guid3dDrawRect(Rect(CursorPos.X, CursorPos.Y,
                        CursorPos.X + 32, CursorPos.Y + 32),
                   gui3dOneColor($FFFFFFFF), uvNormal, 1);

    // Draw black grads.
    guid3dConfigC;
    D3DDEV8.SetTexture(0, NIL);
    guid3dDrawRect(Rect(0, 0, 640, 200),
                   gui3dFourColors($FF000000, $FF000000,
                                   $00000000, $00000000), uvNormal, 1);
    guid3dDrawRect(Rect(0, 280, 640, 480),
                   gui3dFourColors($00000000, $00000000,
                                   $FF000000, $FF000000), uvNormal, 1);
  end else
  begin
    // Draw clouds rect
    guid3dConfigB;
    D3DDEV8.SetTexture(0, Texture1);
    guid3dDrawRect(Rect(200, 100, 440, 380),
                   gui3dFourColors($FFFFFFFF, $FFFFFFFF,
                                   $00FFFFFF, $00FFFFFF), uvNormal, 1);

    // Draw text
    Font2._Begin;
    D3DXFontWrite(Font2, '(Right click to fade and close!)', Rect(240, 450, 640, 480),
                  0, $FFFFFFFF);
    Font2._End;

    // Draw cursor
//    guid3dConfigB;
    D3DDEV8.SetTexture(0, Texture2);
    guid3dDrawRect(Rect(CursorPos.X, CursorPos.Y,
                        CursorPos.X + 32, CursorPos.Y + 32),
                   gui3dFourColors($FFFFFFFF, $80FFFFFF,
                                   $0FFFFFFF, $00FFFFFF), uvNormal, 1);

    // Draw white grads.
    guid3dConfigC;
    D3DDEV8.SetTexture(0, NIL);
    guid3dDrawRect(Rect(0, 0, 100, 480),
                   gui3dFourColors($FFFFFFFF, $00000080,
                                   $FFFFFFFF, $00000080), uvNormal, 1);
    guid3dDrawRect(Rect(540, 0, 640, 480),
                   gui3dFourColors($00000080, $FFFFFFFF,
                                   $00000080, $FFFFFFFF), uvNormal, 1);
  end;

  // Draw fade
  if (CountDown > 0) then
  begin
    _fade := ((FADE_TIME - CountDown) * 255) div FADE_TIME;
    guid3dConfigC;
    D3DDEV8.SetTexture(0, NIL);
    guid3dDrawRect(Rect(0, 0, 640, 480),
                   gui3dOneColor(_fade shl 24), uvNormal, 1);
    dec(CountDown, LASTFRAMETIME);
    if CountDown <= 0 then Application.Terminate;
  end;

  // End the scene.
  D3DDEV8.EndScene;

  // Display scene.
  D3DDEV8.Present(NIL, NIL, 0, NIL);
end;

// -------------------------------------------------------------------

procedure TForm1.AppOnIdle(Sender: TObject; var Done: Boolean);

begin
  Done := false;
  if (not DXReady) or (not WindowReady) then exit;
  GetUserInput;
  RenderFrame;
  // Speed control
  while GetTickCount - FRAMECONTROL < 1 do Sleep(0);
  LASTFRAMETIME := GetTickCount - FRAMECONTROL;
  FRAMECONTROL := GetTickCount;
end;

// ------------------------------------------------------------------

procedure TForm1.AppOnActivate(Sender: TObject);

begin
  WindowReady := true;
end;

// ------------------------------------------------------------------

procedure TForm1.AppOnDeactivate(Sender: TObject);

begin
  WindowReady := false;
end;

// -------------------------------------------------------------------

procedure TForm1.hHalt(_errMsg: pChar);

begin
  MessageBox(Handle, _errMsg, 'Error', MB_ICONERROR);
  Application.Terminate;
end;

// -------------------------------------------------------------------

function GetCustomVertex(_px, _py, _pz, _u, _v: single;
                         _dcolor: dWord): TCustomVertex;

begin
  with Result do
   begin
     Position.x := _px;
     Position.y := _py;
     Position.z := _pz;
     U := _u;
     V := _v;
     DColor := _dcolor;
   end;
end;

// -------------------------------------------------------------------

function InitVertexBuffer: boolean;

var _vb  : array[0..99] of TCustomVertex;
    _cnt : longint;
    _ptr : pByte;
    _theta : single;

begin
  Result := false;

  // Initialize the vertices.
  for _cnt := 0 to 49 do
    begin
      _theta := (2 * PI * _cnt) /49;
      _vb[2 * _cnt] := GetCustomVertex(sin(_theta), -1.0, cos(_theta),
                                       _cnt/49, 1.0, $FFFF7F00);
      _vb[2 * _cnt + 1] := GetCustomVertex(sin(_theta),  1.0, cos(_theta),
                                           _cnt/49, 0.0, $FFFFFFFF);
    end;

  // Create a vertex buffer.
  if failed(D3DDEV8.CreateVertexBuffer(SizeOf(_vb), 0,
                                       D3DFVF_CUSTOMVERTEX,
                                       D3DPOOL_DEFAULT,
                                       VertexBuffer)) then exit;

  // Lock, fill and unlock vertex buffer.
  if failed(VertexBuffer.Lock(0, SizeOf(_vb), _ptr, 0)) then exit;
  Move(_vb, _ptr^, SizeOf(_vb));
  VertexBuffer.Unlock;

  // All right
  Result := true;
end;

// -------------------------------------------------------------------

procedure TForm1.FormActivate(Sender: TObject);

begin
  FRAMECONTROL := GetTickCount;

  CursorPos.X := 320;
  CursorPos.Y := 240;

  // Initialize application events
  Application.OnIdle       := AppOnIdle;
  Application.OnActivate   := AppOnActivate;
  Application.OnDeactivate := AppOnDeactivate;
  Application.OnRestore    := AppOnActivate;
  Application.OnMinimize   := AppOnDeActivate;

  // Initialize DirectInput.
  if not DIInit then begin hHalt(ERR03); exit; end;
  if not DIInitMouse(Handle) then begin hHalt(ERR04); exit; end;

  // Initialize Direct Graphics (first try with Hardware Accel. (HAL))
  if not InitDGfx(Handle, true, 640, 480, 16, 1, true, true) then
    begin
      MessageBox(0, WRN00, 'Warning.', MB_ICONWARNING);
      CloseDGfx;
      if not InitDGfx(Handle, false, 0, 0, 0, 0, false, true) then
        begin hHalt(ERR00); exit; end;
    end;

  // Initialize GUI3D
  if not InitGUI3D then begin hHalt(ERR06); exit; end;

  // Load Textures.
  if failed(D3DXCreateTextureFromFileA(D3DDEV8, TEXTFILENAME1, Texture1)) then
    begin hHalt(ERR02 + TEXTFILENAME1); exit; end;
  Texture2 := D3DXLoadTextureFromFile(TEXTFILENAME2, $FF00FF00, D3DFMT_A1R5G5B5, 1);
  if Texture2 = NIL then begin hHalt(ERR02 + TEXTFILENAME2); exit; end;

  // Create font
  Font1 := D3DXCreateFont('Arial', 30, true, true, true);
  if Font1 = NIL then begin hHalt(ERR05); exit; end;
  Font2 := D3DXCreateFont('Arial', 15, false, false, false);
  if Font2 = NIL then begin hHalt(ERR05); exit; end;

  // Initialize VertexBuffer
  if not InitVertexBuffer then begin hHalt(ERR01); exit; end;

  // Disable culling.
  D3DDEV8.SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);

  // Disable lights.
  D3DDEV8.SetRenderState(D3DRS_LIGHTING, 0);

  // Enable dithering.
  D3DDEV8.SetRenderState(D3DRS_DITHERENABLE, 1);

  // Set texture filtering
  if USE_TEXTURE_FILTERING then
    begin
      D3DDEV8.SetTextureStageState(0, D3DTSS_MAGFILTER, ord(D3DTEXF_LINEAR));
      D3DDEV8.SetTextureStageState(0, D3DTSS_MINFILTER, ord(D3DTEXF_LINEAR));
    end;

  // All ok
  DXReady := true;
end;

// -------------------------------------------------------------------

procedure TForm1.FormDestroy(Sender: TObject);

begin
  DXReady := false;
  VertexBuffer := NIL; // Free Vertex Buffer
  Texture1 := NIL; // Free Texture;
  Texture2 := NIL; // Free Texture;
  Font1 := NIL; // Free font.
  Font2 := NIL; // Free font.
  CloseGUI3D; // Close GUI3D
  CloseDGfx; // Close Direct X Graphics
  DIClose; // Close DirectInput
end;

// -------------------------------------------------------------------

end.
